/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

using UnityEngine;
using UnityEngine.Events;
using UnityEngine.EventSystems;

namespace Completed
{
    /// <summary>
    /// Handles beginning drag, movement while dragging, and release of the drag.
    /// DragEvent allows us to pass a Vector3 through to the event delegate.
    /// In this implementation we're passing the world position of the pointer.
    /// </summary>
    public class DragDropEventHandler : MonoBehaviour, IBeginDragHandler, IDragHandler, IDropHandler
    {

        // Event to fire when dragging begins - assignable in the inspector
        public UnityEvent OnDragStart;

        [System.Serializable] // Mark this as serializable so OnDragMove will show in the inspector.
        public class Vector3Event : UnityEvent<Vector3> { } // Custom UnityEvent with Vector3 parameter.

        // Event to fire when drag is moving - assignable in the inspector
        public Vector3Event OnDragMove;

        // Event to fire when drag stops - assignable in the inspector
        public UnityEvent OnDragReleased;

        // Allows setting of the initial enabled state of this script. enabled == false will
        // prevent method events from firing.
        public bool enabledOnAwake = false;

        private void Awake()
        {
            enabled = enabledOnAwake;
        }

        public void OnBeginDrag(PointerEventData eventData)
        {
            // If this script is disabled then do nothing!
            if (!enabled)
                return;

            // Otherwise fire the UnityEvent we assigned in the inspector for this event.
            OnDragStart.Invoke();
        }

        public void OnDrag(PointerEventData eventData)
        {
            if (!enabled)
                return;

            // Feed the world position of the pointer to OnDragMove
            OnDragMove.Invoke(Camera.main.ScreenToWorldPoint((Vector3)eventData.position + Vector3.forward));
        }

        public void OnDrop(PointerEventData eventData)
        {
            if (!enabled)
                return;

            OnDragReleased.Invoke();
        }
    }
}
