/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

using UnityEngine;

/// <summary>
/// Responsible for 
/// - Moving the enemy via Update()
/// - Allowing access to set the enemy's speed modifier (used in ice patch triggers) via SetSpeedModifier().
/// - Allowing access to destroy this game object via Kill()
/// - Applying damage to the player vis OnCollisionEnter2D()
/// </summary>
public class EnemyController : MonoBehaviour
{

    public Rigidbody2D enemyRigidbody;

    // The normal speed of the enemy.
    public float normalSpeed = 2;

    // This will be used to slow down the enemy when it enters an ice patch trigger.
    private float speedModifier = 1;

    // This is used to prevent the enemy from 'bouncing' into the player's collider
    // and doing damage more than once.
    private bool doDamageOnce = false;

    private void Reset()
    {
        // When this script is added to a game object, automatically attempt to assign the 
        // enemyRigidbody variable.
        enemyRigidbody = GetComponent<Rigidbody2D>();
    }

    private void Update()
    {
        // Move the enemy left (toward the player) at the normal speed times the speed modifier every frame.
        enemyRigidbody.MovePosition(
        (Vector2)enemyRigidbody.transform.position + Vector2.left * normalSpeed * speedModifier * Time.deltaTime);
    }

    // Allows the speedModifier's value to be set via other scripts.
    public void SetSpeedModifier(float value)
    {
        speedModifier = value;
    }

    // Destroys this enemy
    public void Kill()
    {
        Destroy(gameObject);
    }

    // When this enemy collides with the player, reduce the player's health
    // and ensure this is only done once per enemy
    private void OnCollisionEnter2D(Collision2D collision)
    {
        if (collision.gameObject.CompareTag("Player"))
        {
            if (!doDamageOnce)
            {
                doDamageOnce = true;
                Health health = collision.gameObject.GetComponent<Health>();
                if (health)
                    health.health--;
            }
        }
    }
}
