/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

using UnityEngine;
using UnityEngine.Events;

/// <summary>
/// Resposible for maintaining a character's health and, when health is zero, 
/// firing an event that is assignable in the Inspector window.
/// </summary>
public class Health : MonoBehaviour
{

    /// <summary>
    /// Stores the health of the enemy on awake.
    /// Needs to start at 0 so that it won't be overwritten.
    /// </summary>
    private int fullHealth = 0;

    /// <summary>
    /// This can be set via the inspector as the character's initial health.
    /// </summary>
    [SerializeField]
    private int healthValue = 2;

    public int health
    {
        get
        {
            return healthValue;
        }
        set
        {
            healthValue = value;

            if (healthValue == 0)
                OnHealthZero.Invoke();
        }
    }

    // The event to fire when healthValue is zero. This is assigned via the inspector.
    public UnityEvent OnHealthZero;

    // Ensure we store the initial health so that it can be restored.
    private void Awake()
    {
        if (fullHealth == 0)
            fullHealth = healthValue;
    }

    // Use this to restore the health to the initial value. 
    // This is used for the player when the game is reset.
    public void RestoreHealth()
    {
        healthValue = fullHealth;
    }
}
