/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

using UnityEngine;

/// <summary>
/// Responsible for moving and firing the Player's cannon.
/// </summary>
public class PlayerController : MonoBehaviour
{
    // The part of the cannon that will be rotated.
    public GameObject cannon;

    // The min and max angle for the cannon's rotation
    public Vector2 cannonRotationRange = new Vector2(1, 65);

    // How fast the cannon should rotate
    public float rotationSpeed = 20;

    // The original snowball game object that will be cloned when the player fires.
    public Rigidbody2D snowball;

    // The amount of force to apply to the snowball
    public float snowballForce = 3;

    // Use this method to rotate the cannon. Use direction = 1 to rotate up.
    // Use direction = -1 to rotate down.
    public void RotateCannon(int direction)
    {
        // Calculate the new angle.
        float angle = cannon.transform.rotation.eulerAngles.z + direction * rotationSpeed * Time.deltaTime;

        // Ensure the angle is withing the min and max rotation range.
        angle = Mathf.Clamp(angle, cannonRotationRange.x, cannonRotationRange.y);

        // Apply the new rotation along the Z-axis
        cannon.transform.rotation = Quaternion.AngleAxis(angle, Vector3.forward);
    }

    // Use this method to Fire the cannon and launch a snowball.
    public void Fire()
    {
        // Clone the original snowball. We'll cast this as a Rigidbody2D since that's the component we need.
        Rigidbody2D newSnowball =
            Instantiate(snowball, snowball.transform.position, snowball.transform.rotation) as Rigidbody2D;

        // Make sure the snowball will be affected by physics since the prefab is kinematic.
        newSnowball.isKinematic = false;

        // Apply instantaneous force to the snowball. 
        // The cannon's transform.right is used since this provides a vector that matches the cannon's rotation.
        newSnowball.AddForce((Vector2)cannon.transform.right * snowballForce, ForceMode2D.Impulse);
    }
}
