/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

using UnityEngine;

/// <summary>
/// Resposible for water balloon collision behavior.
/// - Plays splash particles
/// - Hides balloon sprite and deactivates its rigidbody
/// - Positions and activates ice patch
/// - Destroys balloon after delay
/// - Ice patches do not appear if Player is hit
/// </summary>
public class WaterBalloonController : MonoBehaviour
{

    // The particles that will play when the water balloon hits an object.
    public ParticleSystem splashParticles;

    // The patch of ice that should be displayed when the baloon hits the ground.
    public GameObject icePatch;

    // The rigidbody on the ballon
    public Rigidbody2D balloonRigidbody;

    // The balloon's sprite renderer
    public SpriteRenderer balloonSpriteRenderer;

    // The balloon's collider
    public Collider2D ballonCollider;

    // When this component is added to a game object, attempt to assign the inspector variables.
    private void Reset()
    {
        balloonRigidbody = GetComponent<Rigidbody2D>();
        balloonSpriteRenderer = GetComponent<SpriteRenderer>();
        ballonCollider = GetComponent<Collider2D>();
    }

    // When the balloon hits something...
    private void OnCollisionEnter2D(Collision2D collision)
    {

        // Prevent any further movement
        balloonRigidbody.isKinematic = true;
        // Prevent any further collisions
        ballonCollider.enabled = false;
        // Hide the balloon sprite
        balloonSpriteRenderer.enabled = false;
        // Start the splash particles
        splashParticles.Play();

        // If the balloon hit something other than the player (should only be the ground)
        // then make the ice patch appear and make it so it is no longer a child
        // so that when we destroy the balloon the ice patch is not destroyed with it.
        if (!collision.gameObject.CompareTag("Player"))
        {
            icePatch.transform.parent = null;
            icePatch.transform.position = collision.contacts[0].point;
            icePatch.SetActive(true);
        }

        // Destroy the balloon and splash particles after the particles have played completely.
        Destroy(gameObject, splashParticles.main.duration + 0.25f);
    }

    // This is used to easily create a copy of this water balloon.
    public WaterBalloonController MakeCopy(Vector3 position)
    {
        return Instantiate(this, position, transform.rotation) as WaterBalloonController;
    }

    // An alternate method for making a copy. This will be called from a UnityEvent via the inspector.
    public void MakeCopy()
    {
        MakeCopy(transform.position);
    }
}
